
/**
 * Creates tabs for a sheet object
 */
 export const createTabs = function(html, tabGroups) {
    // Create recursive activation/callback function
    const _recursiveActivate = function(rtabs, tabName=null) {
      if (tabName == null) this._initialTab[rtabs.group] = rtabs.active;
      else {
        rtabs.activate(tabName);
        this._initialTab[rtabs.group] = tabName;
      }
  
      // Scroll to previous position
      let scrollElems = html.find(`.scroll-${rtabs.group}`);
      if (scrollElems.length === 0) scrollElems = html.find(`.tab[data-group="${rtabs.group}"]`);
      for (let o of scrollElems) { o.scrollTop = this._scrollTab[rtabs.group]; }
  
      // Recursively activate tabs
      for (let subTab of rtabs.subTabs) {
        _recursiveActivate.call(this, subTab, subTab.active);
      }
    };
  
    // Create all tabs
    const _func = function(group, children) {
      if (html.find(`nav[data-group="${group}"]`).length === 0) return null;
  
      if (this._initialTab == null) this._initialTab = {};
      if (this._scrollTab == null) this._scrollTab = {};
  
      const subHtml = html.find(`.${group}-body > div[data-group="${group}"]`);
      const activeSubHtml = subHtml.filter(".active");
      const initial = this._initialTab[group] !== undefined ? this._initialTab[group] : (activeSubHtml.length > 0 ? activeSubHtml[0].dataset.tab : "");
  
      // Set up data for scroll position and active tab
      if (this._scrollTab[group] === undefined) this._scrollTab[group] = 0;
      if (this._initialTab[group] === undefined) this._initialTab[group] = initial;
  
      // Set up scrolling callback
      let scrollElems = html.find(`.scroll-${group}`);
      if (scrollElems.length === 0) scrollElems = html.find(`.tab[data-group="${group}"]`);
      scrollElems.scroll(ev => this._scrollTab[group] = ev.currentTarget.scrollTop);
  
      // Create tabs object
      const tabs = new TabsV2({
        navSelector: `.tabs[data-group="${group}"]`,
        contentSelector: `.${group}-body`,
        callback: (_, tabs) => {
          _recursiveActivate.call(this, tabs);
        },
      });
  
      // Recursively create tabs
      tabs.group = group;
      tabs.subTabs = [];
      for (let [childKey, subChildren] of Object.entries(children)) {
        const newTabs = _func.call(this, childKey, subChildren);
        if (newTabs != null) tabs.subTabs.push(newTabs);
      }
  
      tabs.bind(html[0]);
      _recursiveActivate.call(this, tabs, this._initialTab[group]);
      return tabs;
    };
  
    for (const groupKey of Object.keys(tabGroups)) {
      _func.call(this, groupKey, tabGroups[groupKey]);
    }
  };

export async function importData() {
  
  console.log('DRGH | Starting import')
  const module = game.modules.get("hobgoblin-blockade");
  let scenes = null;
  let actors = [];

  // Create folders for Journals
  let baseFolder = await Folder.create({name: 'Hobgoblin Blockade', type: 'JournalEntry', parent: null});
  let adventureFolder = await Folder.create({name: 'Adventure', type: 'JournalEntry', parent: baseFolder, sorting: "m"});
  let npcFolder = await Folder.create({name: 'Characters', type: 'JournalEntry', parent: baseFolder, sorting: "m"});
  let handoutsFolder = await Folder.create({name: 'Handouts', type: 'JournalEntry', parent: baseFolder, sorting: "m"});

  let importedJournals = new Set();

  for ( let p of module.packs ) {
          if (p.system !== game.system.id && p.system) continue;
          const pack = game.packs.get("hobgoblin-blockade."+p.name);
          if ( p.entity !== "Playlist" && p.entity !== "JournalEntry" ) await pack.importAll();
          else if (p.entity === "Playlist") {
                  const music = await pack.getContent();
                  Playlist.create(music.map(p => p.data));
          } else {
            const journals = await pack.getContent();
            for (let element of journals) {
              let importedElement = await game.journal.importFromCollection(`${p.package}.${p.name}`,element.id);
              if (element.name.startsWith('NPC - '))
                await importedElement.update({folder:npcFolder.id, name:importedElement.name.replace('NPC - ','')})
              else if (element.name.startsWith('Handout - '))
                await importedElement.update({folder:handoutsFolder.id, name:importedElement.name.replace('Handout - ','')})
              else {
                var sortId = parseInt(importedElement.name)
                await importedElement.update({folder:adventureFolder.id, name:importedElement.name.replace(`${sortId}. `,''), sort: sortId})
              }
              importedJournals.add(importedElement)
            }
          }
          if ( p.entity === "Scene" ) scenes = game.folders.getName(p.label);
          if ( p.entity === "Actor" ) actors.push(game.folders.getName(p.label));
  }

  for (let journal of importedJournals) {
    let journalContent = journal.data.content;
    actors.forEach(_directory => {
      _directory.entities.forEach(_actor => {
        if (journalContent)
          journalContent = journalContent.replace(new RegExp(`\\\{${_actor.name}\\\}`,"gi"), `<b>@Actor[${_actor.id}]{${_actor.name}}</b>`)
      
      })
    })
    importedJournals.forEach(_journal => {
      if (journal !== _journal && journalContent) {
        journalContent = journalContent.replace(new RegExp(`\\\[${_journal.name}\\\]`,"gi"), `@JournalEntry[${_journal.id}]{${_journal.name}}`)
      }
    })
    if (journalContent)
      await journal.update({'content':journalContent})
  }

  // Re-associate Tokens for all scenes
  const sceneUpdates = [];
  for ( let s of scenes.entities ) {
          const tokens = duplicate(s.data.tokens).map(t => {
                  for (let actorsDirectory of actors) {
                      const a = actorsDirectory.entities.find(a => a.data?.token?.name === t.name);
                      if (a) {

                              t.actorId = a ? a.id : null;
                              t.actorLink = a ? a.data?.token?.actorLink || false : false;
                              if (a.data?.data?.traits?.size === 'lg') {
                                      t.height = 2;
                                      t.width = 2;
                              } else if (a.data?.data?.traits?.size === 'huge') {
                                      t.height = 3;
                                      t.width = 3;
                              } else if (a.data?.data?.traits?.size === 'med') {
                                      t.height = 1;
                                      t.width = 1;
                              } 
                              console.log('DRGH | Linking actor', a)
                      }
                  }
                  return t;
          });
          sceneUpdates.push({_id: s.id, tokens});
  }
  console.log('DRGH | Linking actor', sceneUpdates)
  await Scene.update(sceneUpdates);


  // Display the introduction
  const j1 = game.journal.getName("BASE JOURNAL NAME");
  if ( j1 ) j1.sheet.render(true, {sheetMode: "text"});
}
